# PumpkinLB Copyright (c) 2014-2015 Tim Savannah under GPLv3.
# You should have received a copy of the license as LICENSE 
#
# See: https://github.com/kata198/PumpkinLB


GRACEFUL_SHUTDOWN_TIME = 6

DEFAULT_BUFFER_SIZE = 4096
